/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ui.TagCloud");

/*
 * Imports 
 */
dojo.require("dojo.i18n"); 

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Slider");

dojo.requireLocalization("jazz.ui", "TagCloudMessages");

/*
 * Documentation:
 * 
 * https://jazz.net/wiki/bin/view/WebUIBook/TagCloud
 */

(function() {

/*
 * Aliases 
 */
var HorizontalSlider = dijit.form.HorizontalSlider;

/*
 * Class 
 *
 * PROVISIONAL: This API is considered provisional and may be changed or removed in the
 * future with minimal or no warning.
 * 
 * @since 0.7
 */
dojo.declare("jazz.ui.TagCloud",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	tags: null,
	viewableTagLimit: 30,
	showFrequencies: false,
	hrefProvider: null,
	onTagClick: null,
	maxTagFontSize: 26,
	minTagFontSize: 12,
	defaultTagFontSize: 12,
	
	// ------------------------------------------------------------------------>
	
	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ui", "templates/TagCloud.html"),
	
	/*
	 * Static Variable Container
	 */
	_static: function() {}, 	

	/*
 	 * Constructor
 	 * 
 	 * @param Object args
 	 * 
 	 * @return TagCloud
 	 */
	constructor: function(args) {
		
		// Messages //------------------------------------------------------------->
		
		this.messages = dojo.i18n.getLocalization("jazz.ui", "TagCloudMessages");
		this.msgLess = this.messages.msgLess;
		this.msgMore = this.messages.msgMore;
		
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {
		// Call super
		this.inherited(arguments);
		
		if (this._validateProperties() === false) {
			// Configuration error
			this._logError(new Error("Specified widget properties are invalid."));
			return;
		}		
		
		this._initializeStaticVariables();
		this._initializeInstanceVariables();
		this._initializeWidget();
		
	},

	/*
 	 * Public Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		delete this._tagNamesSortName;
		delete this._tagValuesSortName;

		delete this._tagNamesSortValue;
		delete this._tagValuesSortValue;

		delete this._maxTagValue;
		delete this._minTagValue;
		delete this._fontSizeRatio;
		delete this._tagCount;
		delete this._tagElements;
		delete this._createTagLinks;

		if (this._tagHandlersOnMouseOver) {
			this._destroyTagHandlers(this._tagHandlersOnMouseOver);
			delete this._tagHandlersOnMouseOver;
		}

		if (this._tagHandlersOnKeyUp) {
			this._destroyTagHandlers(this._tagHandlersOnKeyUp);
			delete this._tagHandlersOnKeyUp;
		}
		
		if (this._tagHandlersOnClick) {
			this._destroyTagHandlers(this._tagHandlersOnClick);
			delete this._tagHandlersOnClick;
		}

		delete this._filterValue;
		delete this._tagDisplayMap;
		
		if (this._horizontalSlider) {
			this._horizontalSlider.destroy();
			delete this._horizontalSlider;
		}

		// Call super
		this.inherited(arguments);
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Object tagHandlers : [String] = Object
 	 * 
 	 * @return void
 	 */		
	_destroyTagHandlers: function(tagHandlers) {
		
		for (var name in tagHandlers) {
			var handler = tagHandlers[name];
			if (!handler) {
				continue;
			}
			this.disconnect(handler);
			delete tagHandlers[name];
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initializeStaticVariables: function() {

		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}
		this._static._instanceCount = 1;

		// Light to dark colours -> Low to high frequencies
		this._static._TAG_LINK_COLOURS = ["#0096CD", "#0084B9", "#005C87"];
		this._static._TAG_TEXT_COLOURS = ["#999999", "#666666", "#333333"];
		
		this._static._USE_DEFAULT = -1;
		
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initializeInstanceVariables: function() {
		
		this._tagNamesSortName = this._extractArray("name");
		this._tagValuesSortName = this._extractArray("value");
		
		this._tagNamesSortValue = this._extractArray("name");
		this._tagValuesSortValue = this._extractArray("value");
		
		this._maxTagValue = Number.MIN_VALUE;
		this._minTagValue = Number.MAX_VALUE;
		this._fontSizeRatio = 0;
		this._colourBucketRange = Number.MAX_VALUE;
		this._tagCount = this.tags.length;
		this._tagElements = new Object();
		
		if (this.hrefProvider) {
			this._createTagLinks = true;
			this._tagHandlersOnMouseOver = new Object();
			this._tagHandlersOnKeyUp = new Object();
		} else if (this.onTagClick) {
			this._createTagLinks = true;
			this._tagHandlersOnClick = new Object();
		} else {
			this._createTagLinks = false;			
		}	
		
		if (this._tagCount > this.viewableTagLimit) {
			this._filterValue = this.viewableTagLimit;
		} else {
			this._filterValue = this._tagCount;
		}		
		
		this._tagDisplayMap = new Object();
		this._horizontalSlider = null;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initializeWidget: function() {
		// Sort tag names alphabetically in descending order
		// Tag name and value indices updated accordingly
		this._tagNamesShellSort(this._tagNamesSortName, 
								this._tagValuesSortName);
		// Sort tag values numerically in descending order
		// Tag name and value indices updated accordingly
		this._tagValuesShellSort(this._tagNamesSortValue, 
								 this._tagValuesSortValue);
		// Setup map to track tag visibility
		this._createTagDisplayMap();
		// Calculate max value, min value and font size ratio
		this._calculateStatistics();
		// Create the tag controls
		this._createControls();		
		// Create the tag cloud
		this._createTagCloud();
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return boolean
 	 */	
	_createTagDisplayMap: function() {
		
		var index;
		var name;
		
		if (this._tagCount > this.viewableTagLimit) {
			for (index = 0; index < this.viewableTagLimit; index++) {
				name = this._tagNamesSortValue[index];
				this._tagDisplayMap[name] = true;
			}			
			for (index = this.viewableTagLimit; index < this._tagCount; index++) {
				name = this._tagNamesSortValue[index];
				this._tagDisplayMap[name] = false;
			}			
		} else {
			for (index = 0; index < this._tagCount; index++) {
				name = this._tagNamesSortValue[index];
				this._tagDisplayMap[name] = true;
			}			
		}		
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return boolean
 	 */	
	_validateProperties: function() {
		
		if ((this.tags instanceof Array) === false) {
			return false;
		} else if (this.tags.length === 0) {
			return false;
		}
		return true;
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return Array
 	 */	
	_getTagColours: function() {
		if (this._createTagLinks) {
			return this._static._TAG_LINK_COLOURS;
		}
		return this._static._TAG_TEXT_COLOURS;
	},	
	
	/*
 	 * @param Number value
 	 * 
 	 * @return String
 	 */	
	_calculateTagColour: function(value) {

		var tagColours = this._getTagColours();
		var lastIndex = tagColours.length - 1;
		var index;	
		
		if (this._colourBucketRange > 0) {
			var normalizedValue = value - this._minTagValue;
			index = Math.floor(normalizedValue / this._colourBucketRange);
			// Index guards (should not be required)
			if (index > lastIndex) {
				index = lastIndex;
			} else if (index < 0) {
				index = 0;
			}
		} else {
			index = lastIndex;
		}
		
		return tagColours[index];
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param Array names
 	 * @param Array values
 	 * 
 	 * @return void
 	 */	
	_tagNamesShellSort: function(names, values) {
		// Shell sort O(n^2)
		// Sort names and values simultaneously
		for (var increment = names.length / 2; increment > 0; increment = (increment == 2 ? 1 : Math.round(increment / 2.2))) {
			for (var i = increment; i < names.length; i++) {
				// Ascending sort
				for (var j = i; j >= increment && names[j - increment] > names[j]; j -= increment) {
					// Values
					var temp1 = values[j];
					values[j] = values[j - increment];
					values[j - increment] = temp1;
					// Names
					var temp2 = names[j];
					names[j] = names[j - increment];
					names[j - increment] = temp2;
				}
			}
		}
	},
	
	/*
 	 * Private Method
 	 * 
 	 * @param Array names
 	 * @param Array values
 	 * 
 	 * @return void
 	 */	
	_tagValuesShellSort: function(names, values) {
		// Shell sort O(n^2)
		// Sort names and values simultaneously
		for (var increment = values.length / 2; increment > 0; increment = (increment == 2 ? 1 : Math.round(increment / 2.2))) {
			for (var i = increment; i < values.length; i++) {
				// Descending sort
				for (var j = i; j >= increment && values[j - increment] < values[j]; j -= increment) {
					// Values
					var temp1 = values[j];
					values[j] = values[j - increment];
					values[j - increment] = temp1;
					// Names
					var temp2 = names[j];
					names[j] = names[j - increment];
					names[j - increment] = temp2;
				}
			}
		}
	},	
	
	/*
 	 * @param String propertyName
 	 * 
 	 * @return Array
 	 */	
	_extractArray: function(propertyName) {
		var newArray = new Array();
		
		for (var index = 0; index < this.tags.length; index++) {
			newArray[index] = this.tags[index][propertyName];
		}				
		
		return newArray;
	},	
	
	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_calculateStatistics: function() {
		
		// Calclulate max tag value
		this._maxTagValue = this._tagValuesSortValue[0];

		// Calculate min tag value
		this._minTagValue = this._tagValuesSortValue[this._tagCount - 1];
		
		// Calculate font size ratio
		var fontSizeDifference = this.maxTagFontSize -
								 this.minTagFontSize;
		var valueDifference = this._maxTagValue - 
							  this._minTagValue;
		
		if (valueDifference > 0) {
			this._fontSizeRatio = fontSizeDifference / valueDifference;
			this._colourBucketRange = (valueDifference + 1) / this._getTagColours().length;
		} else {
			// The min and max tag values are the same.
			// Make all tags have equal font weight and colour by using the default.
			this._fontSizeRatio = this._static._USE_DEFAULT;
			this._colourBucketRange = this._static._USE_DEFAULT;
		}		
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_createTagCloud: function() {
		
		// Create tags		
 		for (var index = 0; index < this._tagCount; index++) {
			
			var name = this._tagNamesSortName[index];
			var value = this._tagValuesSortName[index];
			var show = this._tagDisplayMap[name];
					
			var tagElement = this._createTag(name, value, show);
											 
			this._tagElements[name] = tagElement;
													 
			this._tagArea.appendChild(tagElement);	
		}
	},		

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_createControls: function() {
		// Add the tag filter control
		if (this._tagCount > 1) {
			// At least two tags must exist in order for this control to be 
			// at all useful. Also, on IE, an alert exception occurs
			// "Error: Invalid argment" if only one tag exists.
			this._createTagFilterControl();
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_createTagFilterControl: function() {
		// Create horizontal slider
		this._horizontalSlider = new HorizontalSlider({
				onChange: dojo.hitch(this, this._handleEventFilterValueChanged),
				minimum: 1,
				maximum: this._tagCount,
				discreteValues: this._tagCount,
				intermediateChanges: true,
				value: this._filterValue
			});
		
		// Assemble contents	
		this._tagFilterControl.appendChild(this._horizontalSlider.domNode);		
		// Make visible
		this._controlArea.style.display = "block";
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Number newValue
 	 * 
 	 * @return void
 	 */		
	_handleEventFilterValueChanged: function(newValue) {
		
		var show;
		var targetIndex;
		
		if (newValue > this._filterValue) {
			// Show more action
			show = true;
			// Exclusive value
			targetIndex = newValue - 1;
		} else if (newValue < this._filterValue) {
			// Show less action
			show = false;
			// Inclusive value
			targetIndex = newValue;
		} else {
			// No change. Nothing to do
			return;
		}
		
		// Calculate intermediate values
		// The slider skips over ranges if slid rapidly
		var delta = newValue - this._filterValue;
		var direction;
		if (delta > 0) {
			// Negative number = less direction
			direction = -1;
		} else {
			// Positive number = more direction
			direction = 1;
		}
		
		// Update visiblity of tags within intermediate range
		for (var offset = 0; offset < Math.abs(delta); offset++) {
			var index = targetIndex + (offset * direction);
			
			var name = this._tagNamesSortValue[index];
			var element = this._tagElements[name];
			this._tagDisplayMap[name] = show;
			this._updateNodeVisibility(element, show);			
		}
		
		// Update current value for next event
		this._filterValue = newValue;
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Element node
 	 * @param Boolean show
 	 * 
 	 * @return void
 	 */	
	_updateNodeVisibility: function(node, show) {
		if (show) {
			node.style.display = "inline";	
		} else {
			node.style.display = "none";	
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String name
 	 * @param String value
 	 * @param Boolean show
 	 * 
 	 * @return Element
 	 */	
	_createTag: function(name, value, show) {
		
		// Container
		var container = document.createElement("span");
		dojo.addClass(container, "tag");
		container.style.fontSize = this._calculateFontSize(value);
		this._updateNodeVisibility(container, show);		
		
		// No Break
		// Disallow breaking between the name and frequency
		var nobreakElement = document.createElement("nobr");		
		
		// Name
		var tagNameNode;
		if (this._createTagLinks) {
			tagNameNode = this._createTagLink(name);
			tagNameNode.style.color = this._calculateTagColour(value);
		} else {
			tagNameNode = document.createTextNode(name);
			container.style.color = this._calculateTagColour(value);
		}
		
		// Frequency
		var frequencyElement;
		if (this.showFrequencies) {
			frequencyElement = document.createElement("span");
			var frequencyTextNode = document.createTextNode("(" + value + ")");
			dojo.addClass(frequencyElement, "frequency");
			frequencyElement.appendChild(frequencyTextNode);
		} else {
			// Show frequency as a hovertip
			container.title = value;
		}
		
		// Assemble
		nobreakElement.appendChild(tagNameNode);
		if (this.showFrequencies) {
			nobreakElement.appendChild(frequencyElement);
		}
		container.appendChild(nobreakElement);
		// Create whitespace in order to allow breaking between tags
		container.appendChild(document.createTextNode(" "));

		return container; 			
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Number value
 	 * 
 	 * @return String
 	 */		
	_calculateFontSize: function(value) {
		var fontSize;
		if (this._fontSizeRatio < 0) {
			// Edge case
			// The list values are all of equal weight
			fontSize = this.defaultTagFontSize;
		} else {
			var scaledValue = (this._maxTagValue - value) * 
							  this._fontSizeRatio;
			fontSize = Math.round(this.maxTagFontSize - scaledValue);			
		}

		return fontSize + "px";
	},

	/*
 	 * Private Method
 	 * 
 	 * @param String name
 	 * 
 	 * @return Element
 	 */	
	_createTagLink: function(name) {

		var tagNameNode = document.createTextNode(name);	
		var linkElement = document.createElement("a");

		// Set a default href value in order to enable keyboard accessibility
		linkElement.href = "#";
		linkElement.appendChild(tagNameNode);
	
		if (this.hrefProvider) {
			// Handle mouse over events
			this._tagHandlersOnMouseOver[name] = 
				this.connect(linkElement, "onmouseover", this._handleEventOnMouseOverTag);			
			// Handle key up events	
			this._tagHandlersOnKeyUp[name] = 
				this.connect(linkElement, "onkeyup", this._handleEventOnKeyUpTag);				
		} else if (this.onTagClick) {
			// Handle click events
			this._tagHandlersOnClick[name] = 
				this.connect(linkElement, "onclick", this._handleEventOnClickTag);
		}
		return linkElement;				
	},	

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * 
 	 * @return void
 	 */	
	_handleEventOnMouseOverTag: function(event) {
		dojo.stopEvent(event);
		
		// Get target
		var target = event.target;
		if (!target) {
			return;
		} else if (!target.firstChild) {
			return;
		} else if (!target.firstChild.nodeValue) {
			return;
		}
		var tagName = event.target.firstChild.nodeValue;
		// Get the link url
		var href = this.hrefProvider(this, tagName);
		// Set the link url
		target.href = href;
		
		// Remove mouse over handler
		if (this._tagHandlersOnMouseOver[tagName]) {
			this.disconnect(this._tagHandlersOnMouseOver[tagName]);
			delete this._tagHandlersOnMouseOver[tagName];
		}
		// Remove key up handler
		if (this._tagHandlersOnKeyUp[tagName]) {
			this.disconnect(this._tagHandlersOnKeyUp[tagName]);
			delete this._tagHandlersOnKeyUp[tagName];
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * 
 	 * @return void
 	 */	
	_handleEventOnKeyUpTag: function(event) {
		if (event &&
			(event.keyCode === dojo.keys.TAB)) {
			this._handleEventOnMouseOverTag(event);
		}
	},

	/*
 	 * Private Method
 	 * 
 	 * @param Event event
 	 * 
 	 * @return void
 	 */	
	_handleEventOnClickTag: function(event) {
		dojo.stopEvent(event);
		
		// Get target
		var target = event.target;
		if (!target) {
			return;
		} else if (!target.firstChild) {
			return;
		} else if (!target.firstChild.nodeValue) {
			return;
		}
		var tagName = event.target.firstChild.nodeValue;
		
		// Execute callback
		this.onTagClick(this, tagName);
	},
	
	/*
 	 * Public Method
 	 * 
 	 * @param String tagName
 	 * 
 	 * @return Number
 	 */	
	getTagValue: function(tagName) {
		// Binary Search
		var low = 0;
		var high = this.tags.length - 1;
		var mid;
		
		while (low <= high) {
			mid = Math.floor((low + high) / 2);
			if (this._tagNamesSortName[mid] > tagName) {
				high = mid - 1;
			} else if (this._tagNamesSortName[mid] < tagName) {
				low = mid + 1;
			} else {
				// Tag name found
				return this._tagValuesSortName[mid];
			}
		}

		// Tag name NOT found
		return -1;
	},	
	
	/*
 	 * @param Error
 	 * 
 	 * @return void
 	 */
	_logError: function(errorObject) {
		
		var className = "";
		if (this.declaredClass) {
			className = this.declaredClass + ": ";
		} 
		
		var lineNumber = "";
		if (errorObject.lineNumber) {
			lineNumber = "(line " + errorObject.lineNumber + "): ";
		} 
		
		var errorMessage = "[ERROR]: " +
							className +
							lineNumber +
							errorObject.message;
							
		console.error(errorMessage);
	}	
	
});

})();
