/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("net.jazz.ajax.core.internal.UrlListener");

dojo.require("net.jazz.ajax.core.internal.IEUriMonitor");

var IEUriMonitor  = net.jazz.ajax.core.internal.IEUriMonitor;

dojo.declare("net.jazz.ajax.core.internal.UrlListener", null, {
	// Summary:
	//		Monitors the browser's location (URI) for changes. If a change is
	//		detected, "jazz.onhashchange" event is published.
	// How to use:
	//		Require this package with a dojo-require, then subscribe:
	// 			dojo.subscribe("jazz.onhashchange", this, this.hashChange);
	// 		hashChange will get the new hash value as its first param.
	
	// Do not instantiate this object with the "new" operator - just subscribe to the event
	
	constructor: function(args) {
		if (dojo.getObject("net.jazz.ajax.core.internal._urlListener")) {
			console.error("UrlListener is self-instantiating. Do not call the constructor directly.")
			return;
		}
		//store current hash value
		this.recentLocationHash = window.location.hash.substring(1);
		
		if (dojo.isIE >= 8) {
			return;
		} else {
			document.body.onhashchange = function() { };
			
			if (dojo.isIE && dojo.isIE <= 7) {
				//Use hidden iframe and black magic in older versions of IE
				this.IEUriMonitor = new IEUriMonitor(dojo.hitch(this, this._onHashChange));
			} else {
				//Poll the window location every 200 ms in other browsers.
				setInterval(dojo.hitch(this, this._pollLocation), 200);
			}
		}
	},
	_pollLocation: function() {
		if(window.location.hash.substring(1) === this.recentLocationHash) {
			return;
		}
		this.recentLocationHash = window.location.hash.substring(1);
		this._hashChangeNotify(this.recentLocationHash);
	},
	_onHashChange: function() {
		var hash = window.location.hash.substring(1);
		this._hashChangeNotify(hash);
	},
	_hashChangeNotify: function(hash) {
		this.recentLocationHash = hash;
		if (document.body.onhashchange)
			document.body.onhashchange.call(window);
	}
});

if (!dojo.getObject("net.jazz.ajax.core.internal._urlListener")) {
	//Instantiate myself and store in a global variable
	dojo.setObject("net.jazz.ajax.core.internal._urlListener", new net.jazz.ajax.core.internal.UrlListener({}));
}
})();