/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureAdvanced");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");

dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyComponent");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ConfigurePageMessages");

var checkNotNull 	        = net.jazz.ajax.toolbox.lang.checkNotNull;
var ConfigUtility			= com.ibm.team.repository.web.ui.internal.admin.server.ConfigUtility;
var RepositoryClient		= com.ibm.team.repository.web.client.internal.RepositoryClient;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ServerPropertyComponent = com.ibm.team.repository.web.ui.internal.admin.server.ServerPropertyComponent;
var ServerStatusClient 		= com.ibm.team.repository.web.client.internal.ServerStatusClient;
var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ConfigureAdvanced", [ConfigUtility, dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ConfigureAdvanced.html"),
	enableEdits: true,

	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ConfigurePageMessages");
	},
	
	// widget initialization
	postCreate: function() {
		this.header = new WizardStepHeader({ 
			stepTitle: this.messages.advancedHeader,
			reset: dojo.hitch(this,"refresh")
		});
		this.headerArea.appendChild(this.header.domNode);
		this.inherited(arguments);
		this._initializeContent();
	},

	refresh: function () {
		this.header.clearMessages();
	},
	
	_initializeContent: function() {
		this._showLoadingMsg();
		var srh = new ServiceResponseHandler(this, "_handleComponentsInfo", "_handleComponentsFetchError");
		ServerStatusClient.getComponentsWithServiceConfig(srh);
		this._saveAllButton.disabled = true;
		this._createSaveHandler(this._saveAllButton);
	},
	
	/*
	 * componentInfo is an object that represents the entire set of components, services, and properties installed on the server
	 * 
	 * This function will create a set of ServerPropertyComponents.  Each ServerPropertyComponent will be given all the information
	 * necessary to create its child services and elements.
	 */
	_handleComponentsInfo: function (componentInfo) {
		this.enableEdits = true;
		this._initFilters();
		this._components = new Array();
		if (componentInfo != null) {
			this._serverPropertiesDiv.style.display = "none";
			for (var i=0; i < componentInfo.length; i++) {
				var curComponent = new ServerPropertyComponent({
					"componentInfo" : componentInfo[i].componentInfo,
					"servicesInfo" : componentInfo[i].services,
					"extensionServicesInfo" : componentInfo[i].extensionServices,
					"enableEdits": this.enableEdits,
					"serviceHeaderClass": ["header2"],
					"componentHeaderClass": ["header1","largeHeader"],
					"propertiesRowClass": [],
					"displayIn": this._serverPropertiesDiv
				});
				this._components.push(curComponent);
				this.connect(curComponent, "_show", "_hideEmptyListMsg");
				this.connect(curComponent, "_onServiceResponse", "checkInitialized");
				this.connect(curComponent, "_hide", "_checkShowEmptyListMsg");
				this.connect(curComponent, "_markChanged", function () {
					this._saveAllButton.disabled = false;
				});
				this.connect(curComponent, "_checkUnchanged", function () {
					var change = false;
					for (var j=0; j < this._components.length; j++) {
						if (this._components[j]._changed) {
							change = true;
							break;
						}
					}
					if (!change) {
						this._saveAllButton.disabled = true;
					}
				});
			}
		}
	},
	
	checkInitialized: function () {
		var count = 0;
		for (var i=0; i < this._components.length; i++) {
			count += this._components[i].getPendingServiceCalls();
		}
		if (count == 0) {
			this._initialized = true;
			this._hideDefault.disabled = false;
			this._hideUnchanged.disabled = false;
			this._serverPropertiesDiv.style.display = "";
			this._hideLoadingMsg();
		}
	},
	
	_handleComponentsFetchError: function (errorObj) {
		this.header.displayServiceCallbackErrorMessage(errorObj);
	},
	
	_initFilters: function () {
		this._hideDefault._base = this;
		this._hideDefault.filter = {
			name: "hideDefault",
			method: function (elem) {
				if (elem._editValueNode.value == elem._savedValue && (elem._editValueNode.value == elem._defaultValue || 
					(elem._editValueNode.value == "" && elem._defaultValue == null))) {
					return true;
				}
				else {
					return false;
				}
			}
		}
		this._hideDefault.onclick = function () {
			if (this.checked) {
				this._base._applyFilter(this.filter);
			}
			else {
				this._base._removeFilter(this.filter);
			}
		}
		
		this._hideUnchanged._base = this;
		this._hideUnchanged.filter = {
			name: "hideUnchanged",
			method: function (elem) {
				if (elem._editValueNode.value == elem._savedValue) {
					return true;
				}
				else {
					return false;
				}
			}
		}
		this._hideUnchanged.onclick = function () {
			if (this.checked) {
				this._base._applyFilter(this.filter);
			}
			else {
				this._base._removeFilter(this.filter);
			}
		}
	},
	
	_applyFilter: function (filter) {
		for (var i=0; i < this._components.length; i++) {
			for (var j=0; j < this._components[i]._services.length; j++) {
				for (var k=0; k < this._components[i]._services[j]._properties.length; k++) {
					if (filter.method(this._components[i]._services[j]._properties[k])) {
						this._components[i]._services[j]._properties[k]._hide();
						this._components[i]._services[j]._properties[k]._activeFilters[filter.name] = true;
					}
				}
			}
		}
	},
	
	_removeFilter: function (filter) {
		for (var i=0; i < this._components.length; i++) {
			for (var j=0; j < this._components[i]._services.length; j++) {
				for (var k=0; k < this._components[i]._services[j]._properties.length; k++) {
					if (this._components[i]._services[j]._properties[k]._activeFilters[filter.name]) {
						delete this._components[i]._services[j]._properties[k]._activeFilters[filter.name];
						var empty = true;
						for (key in this._components[i]._services[j]._properties[k]._activeFilters) {
							empty = false;
							break;
						}
						if (empty) {
							this._components[i]._services[j]._properties[k]._show();
						}
					}
				}
			}
		}
	},
	
	_checkShowEmptyListMsg: function () {
		var showMsg = true;
		for (var i=0; i < this._components.length; i++) {
			if (this._components[i]._showing) {
				showMsg = false;
				break;
			}
		}
		if (showMsg) {
			this._showEmptyListMsg();
		}
	},
	
	_showLoadingMsg: function () {
		this._loadingMsg.style.display = "";
	},
	
	_hideLoadingMsg: function () {
		this._loadingMsg.style.display = "none";
	},
	
	_showEmptyListMsg: function () {
		if (this._initialized) {
			this._emptyPropertyListMsg.style.display = "";
		}
	},
	
	_hideEmptyListMsg: function () {
		this._emptyPropertyListMsg.style.display = "none";
	},

	_getChanges: function () {
		var changes = new Array();
		var properties = new Array();
		for (var i = 0; i < this._components.length; i++) {
			if (this._components[i]._changed) {
				var changeObj = this._components[i]._getChanges();
				changes = changes.concat(changeObj["changes"]);
				properties = properties.concat(changeObj["properties"]);
			}
		}
		return {
			"changes": changes,
			"properties": properties
		};
	},
	
	_disableEdits: function () {
		this.enableEdits = false;
		dojo.toggleClass(this._serverPropertiesDiv, "disableEditMode", true);
		if (this._components != null) {
			for (var i=0; i < this._components.length; i++) {
				if (this._components[i]._services != null) {
					for (var j=0; j < this._components[i]._services.length; j++) {
						this._components[i]._services[j]._toggleEdit(false);
					}
				}
			}
		}
	}
});

})();