/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.CrossServerCommunication");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardForm");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.ui.Dialog");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");


(function() {
var PACKAGE_PREFIX = "com.ibm.team.repository.web.ui.internal.admin.server.";
var messages;
var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;
var WizardForm = com.ibm.team.repository.web.ui.internal.wizard.WizardForm;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var REMOVE_ICON_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove.gif");

/*
 * Jazz Foundation 1.0 version of this code:
 * - asked the user to supply the JFS server and rootservices
 * - requested provisional key from JFS server input with hard-coded URI
 * - stored the input for JFS server as the OAuth domain (single entry)
 * 
 * Changes for 1.0.0.1:
 * - don't ask for JFS server
 * - look up the OAuth domain and provisional URI from root services
 * - use the retrieved provisional URI to request provisional key
 * - store the data retrieved from root services as the OAuth domain list
 * 
 */

dojo.declare(PACKAGE_PREFIX + "CrossServerCommunication", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/SetupOAuth.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
		this.approveOAuthMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");
		this.stepTitle = messages.stepTitle; 
	},
	
	postCreate: function () {
		this.header =  new WizardStepHeader({stepTitle: this.stepTitle, reset: dojo.hitch(this,"reload")});
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		var constructor = dojo.getObject(PACKAGE_PREFIX + "OAuthRequest");
		var args = {
			formButton: this.requestButton,
			setStatusCallback: dojo.hitch(this, this.setFormStatus), 
			templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/SimpleWizardForm.html")
		};
		this.oauthForm = new constructor(args);
		this.oauthRequestArea.appendChild(this.oauthForm.domNode);
		this.reload();
	}, 
	
	reload: function () {
		this.header.clearMessages();
		dojo.toggleClass(this.provisionalStepArea, "noDisplay", true);
		this.oauthForm.clearData();
		this.oauthForm.disableFields(false);
		this.reloadFriends();
	},
	
	reloadFriends: function () {
		dojo.toggleClass(this.noFriends, "noDisplay", true);
		dojo.toggleClass(this.friendsTable, "noDisplay", true);
		dojox.data.dom.removeChildren(this.friendsTableBody);	
		this.getFriends();
	},
		
	errorCallback: function (errorObj, contextMsg) {
		this.header.displayServiceCallbackErrorMessage(errorObj, contextMsg);
		this.oauthForm.disableFields(false);
	},
		
	acquiredProvisionalKey: function(key) {
		this.provisionalKey = key.key;
		this.storeProvisionalKey();	

		var newKey = document.createElement("div");
		newKey.className = "spaced-all";
		var needsAuth = dojo.string.substitute(this.messages.keyNeedsAuthorization, [key.key]);
		newKey.innerHTML = needsAuth;
		this.provisionalContentArea.appendChild(newKey);
		
		dojo.toggleClass(this.provisionalStepArea, "noDisplay", false);

		var self = this;
		var provisionalKeyHandler = {
			success: function (approvalUri) {
				if (!approvalUri) {
					// The other server might not have the approval URI in its root services.
					// Do nothing if we can't offer the approval link.
					return;
				}
				var newLink = document.createElement("div");
				newLink.className = "spaced-all border";
				key.server = self.oauthForm.getServerFromRootServices();
				var approvalLink = "<a id='approvelink" + key.key + "' style='cursor:pointer;'>";
				var needsAuth = dojo.string.substitute(self.messages.authorizationLink, [key.server, approvalLink, "</a>"]);
				newLink.innerHTML = needsAuth;
				self.provisionalContentArea.appendChild(newLink);
				
				var approveElemLink = dojo.byId("approvelink" + key.key);
				dojo.connect(approveElemLink,"onclick",self,function(e) {
					dojo.stopEvent(e);
					self._launchApprovalDialog(key, approvalUri);
				});
			},
			failure: function (errorObj) {
				// The other server might not have the approval URI in its root services.
				// Do nothing if we can't offer the approval link.
			}
		};
		var rootServicesURL = this.oauthForm.getRootServices();
		OAuthAdminClient.getFriendsOauthApprovalUri(new SRH(provisionalKeyHandler, "success", "failure"), {url: rootServicesURL});
	}, 
		
	_launchApprovalDialog: function (key, approvalUri) {	
		var source = approvalUri + "?key=" + key.key;
		var iframeParent = document.createElement("div")	
		
		var boxArgs = {
			contentNode: iframeParent, 
			primaryTitle: key.server + " | " + this.approveOAuthMessages.moduleHeading, 
			height: "400px"
		};
		this.viewBox = new jazz.ui.Dialog(boxArgs);
		
		var iframe = document.createElement("iframe");
		iframe.src = source;
		iframe.marginWidth="0"
		iframe.marginHeight="0";
//		iframe.scrolling="no";
		iframe.style.width = "100%";
		iframe.style.height = "390px";
		iframe.style.border = "none";
		iframeParent.appendChild(iframe);
	},
	
	errorRequestingKey: function (errorObj) {
		var serverUrl = this.oauthForm.getRootServices();
		var contextMsg = dojo.string.substitute(this.messages.requestKeyError, [serverUrl]);
		this.errorCallback(errorObj, contextMsg)
	},

	storeProvisionalKey: function() {
		var data = this.oauthForm.getData();
		var args = {
			title: data.title, 
        	rootServices: data.rootServices,
        	oauthConsumerKey: this.provisionalKey,
        	oauthConsumerSecret: data.consumerSecret
		};
		OAuthAdminClient.postNewFriend(new SRH(this, "successStoredProvisionalKey", "errorPostingFriend"), args);	
	},
	
	successStoredProvisionalKey: function () {
		this.provisionalKey = null;
		this.header.clearMessages();
		this.oauthForm.disableFields(false);
		this.reloadFriends();
	},
	
	errorPostingFriend: function (errorObj) {
		this.errorCallback(errorObj, this.messages.postFriendError);
	},
	
	setFormStatus: function (status) {
		if (status === -1) {
			this.header.clearMessages();
		}
		else {
			this.header.showStatus({
				summary: status.message,
				type: status.type
			});
		}
	}, 
	
	onRequestButtonClick: function () {
		this.requestButton.disabled = true;
		this.oauthForm.disableFields(true);
		this.header.showStatus({summary: this.messages.requestingAccess, type: WizardStepHeader.STATUS_TYPES.INFO});
		this.testNewFriendProperties();
	}, 
	
	testNewFriendProperties: function() {
		var data = this.oauthForm.getData();
		
		var args = {
			title: data.title, 
        	rootServices: data.rootServices, 
			oauthConsumerKey: "TEST_KEY", 
			oauthConsumerSecret: "TEST_SECRET"
		};
		// errorPostingFriend is a close enough message for a server error since we shouldn't add new strings 
		OAuthAdminClient.testNewFriend(new SRH(this, "successTestingNewFriendProperties", "errorPostingFriend"), args);	
	},
	
	successTestingNewFriendProperties: function (result) {
		if (result.code && result.code === "OK") {
			this.fetchOauthProvisionalRequestUri();
		} else {
			var status = {
				type: result.code,
				summary: result.message
			}
			if (result.stacktrace) {
				if (dojo.isArray(result.stacktrace)) {
					var detail = result.stacktrace.join("<br>");
				} else {
					detail = result.stacktrace;
				}
				status.details = [{message: detail, code: result.code}];
			}
			this.header.showStatus(status);
			this.oauthForm.disableFields(false);
		}
	},
	
	fetchOauthProvisionalRequestUri: function () {
		var rootServicesURL = this.oauthForm.getRootServices();
		OAuthAdminClient.getOauthProvisionalRequestUri(new SRH(this, "requestProvsionalKey", "errorFetchingRootServices"), {url: rootServicesURL});
	},

	errorFetchingRootServices: function(errorObj) {
		this.errorCallback(errorObj, this._getfetchRootServicesErrorMsg());
	},
	
	_getfetchRootServicesErrorMsg: function () {
		var serverUrl = this.oauthForm.getRootServices();
		var contextMsg = dojo.string.substitute(this.messages.fetchRootServicesError, [serverUrl]);
		return contextMsg;
	},
				
	requestProvsionalKey: function(oauthRequestUrl) {
		if (!oauthRequestUrl) {
			this.header.showStatus({summary: this._getfetchRootServicesErrorMsg(), type: WizardStepHeader.STATUS_TYPES.ERROR});
			this.oauthForm.disableFields(false);
			return;
		}
		var args = this.oauthForm.getDataObject();
		args.url = oauthRequestUrl;
		OAuthAdminClient.requestRemoteProvisionalKey(new SRH(this, "acquiredProvisionalKey", "errorRequestingKey"), args);
	},
	
	getFriends: function () {
		var srh = new SRH(this, "displayFriends", "errorFetchingFriends")
		var args = {includeInternal: true};
		OAuthAdminClient.getFriends(srh,args);
	},
	
	displayFriends: function (friends) {
		dojo.toggleClass(this.friendsArea, "noDisplay", false);
		
		if (friends){
			this.friendList = friends;
			if(this.friendList.length > 1) {
				this.sortFriends();
			}
			var self = this;
			dojo.forEach(this.friendList, function(friend){
                self.addFriendListRow(friend);
            });
		}
		
		dojo.toggleClass(this.noFriends, "noDisplay", friends);
		dojo.toggleClass(this.friendsTable, "noDisplay", !friends);
	},
	
	errorFetchingFriends: function (errorObj) {
		this.errorCallback(errorObj, this.messages.fetchFriendsError);
	},
	
	sortFriends: function () {	
		return this.friendList.sort(function(a, b){
			var aTitle = a.title.toLowerCase();
			var bTitle = b.title.toLowerCase();
			return aTitle > bTitle ? 1 : -1;
		});
	},
	
	addFriendListRow: function(friend) {
		var row = document.createElement("tr");
		var deleteParam = friend.oAuthConsumerKey;
		row.setAttribute("id","row-" + deleteParam);
		
		var cell1 = document.createElement("td");
		var title = this._getEditFriendLink(friend);
		cell1.appendChild(title);		
		row.appendChild(cell1);

		var cell2 = document.createElement("td");
		cell2.innerHTML = friend.rootServicesURL;
		row.appendChild(cell2);
		
		var cell3 = document.createElement("td");
		cell3.innerHTML = friend.oAuthConsumerKey;
		row.appendChild(cell3);
		
		var cell3B = document.createElement("td");
		cell3B.innerHTML = friend.internal ? this.messages.internalLabel : this.messages.internalNo;
		row.appendChild(cell3B);
			
		var cell4 = document.createElement("td");
		
		//Build the Delete Button
        var del = document.createElement("img");
        del.setAttribute('src', REMOVE_ICON_SRC);
		del.setAttribute('id', "del-" + deleteParam);
		del.setAttribute('title', this.messages.removeFriendButton);
        
		cell4.appendChild(del);
		row.appendChild(cell4);
		
		this.friendsTableBody.appendChild(row);
		dojo.connect(del,"onclick",this, "_deleteFriend");
		dojo.addClass(del,"delButton");
	}, 
	
	_deleteFriend: function(e) {
		var verifyDelete = confirm(this.messages.confirmDelete);
        if (verifyDelete) {
			var key = e.target.id;
			key = key.substring(4); 
			this._keyPendingDeletion = key;
			var friend = this._getFriendForKey(key);
			if (friend) {
				var title = friend.title;
				OAuthAdminClient.deleteFriend(new SRH(this, "removeFriendEntry", "errorDeletingFriend"), {title: title});
			}
		}
	},
	
	_getFriendForKey: function(key) {
		if (this.friendList) {
			for (friend in this.friendList) {
				if (this.friendList[friend].oAuthConsumerKey === key) {
					return this.friendList[friend];
				}
			}
		}
		return null;
	},
	
	removeFriendEntry: function() {
		var elem = dojo.byId("row-" + this._keyPendingDeletion);
		this.friendsTableBody.removeChild(elem);
		if (this.friendsTableBody.childNodes.length == 0) {
			dojo.toggleClass(this.noFriends, "noDisplay", false);
			dojo.toggleClass(this.friendsTable, "noDisplay", true);
		}
	}, 
	
	errorDeletingFriend: function (errorObj) {
		var contextMsg = dojo.string.substitute(this.messages.deleteFriendError, [this._keyPendingDeletion]);
		this.errorCallback(errorObj, contextMsg);
	}, 
	
	_getEditFriendLink: function (friend) {
		var aElement = document.createElement("a");
		aElement.onclick = dojo.hitch(this, function() {
			this._launchEditFriend(friend);
		});
		aElement.className ="nameLink";
		aElement.innerHTML = friend.title;
		return aElement;
	},
	
	_launchEditFriend: function (friend) {
		var caller = this;
		var step1 = new com.ibm.team.repository.web.ui.internal.admin.server.EditFriendStep({
			stepTitle: this.messages.editFriendDialogTitle,
			friend: friend, 
			caller: caller
		});

		var wizardArgs = {
			steps: [step1],
			primaryTitle: dojo.string.substitute(this.messages.editFriendHeading, [friend.title]),
			showStepSummary: false,
			processFinish: function(){
				this._steps[0].saveUpdates();
			},
			processCancel: function() {		
				if (this._steps[0].hasUpdates()) {
					var verifyCancel = confirm(caller.messages.confirmEditCancel)
					if (verifyCancel) {
						this.container.viewBox.close();
					}
				} else {
					this.container.viewBox.close();
				}				
			}, 	
			saveSuccess: function () {
				caller.reloadFriends();
				caller.header.clearMessages();
				this.container.viewBox.close();
			}
		}
		this.wizard = new com.ibm.team.repository.web.ui.wizard.Wizard(wizardArgs);
					
	}
	
});

var FORM_HEADER_DEF = [
	{type: "label", labelKey: "propertyLabel"},
	{type: "data", labelKey: "valueLabel"},
	{type: "description", labelKey: "descriptionLabel"}
];
var OAUTH_REQUEST_TABLE = {
	header: FORM_HEADER_DEF,
	rows: [
		{name: "title", labelKey: "titleLabel", descriptionKey: "titleDescrption"},
		{name: "rootServices", labelKey: "rootServicesLabel", descriptionKey: "rootServicesDescrption", exampleKey: "rootServicesExample"},
		{name: "consumerSecret", labelKey: "secretLabel", type: "password", descriptionKey: "secretDescrption"},
		{name: "repeatSecret", labelKey: "secretLabel2", type: "password", descriptionKey: "secretDescrption2", mustMatch: {
			matchName: "consumerSecret",
			errorMessageKey: "secretMismatch"
		}},
		{name: "trusted", labelKey: "trustedLabel", type: "checkbox", descriptionKey: "trustedDescription"}
	]
};

dojo.declare(PACKAGE_PREFIX + "OAuthRequest", WizardForm, {
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
		this.heading =  "";
		this.step = "";
		this.description = "";
	},
	
	postCreate: function () {
		this.inherited(arguments);
		this.validate();
	},
	
	tableContent: OAUTH_REQUEST_TABLE,
		
	getDataObject: function() {
		var valueMap = this.getData();
		var args = new Object();
		for (i in valueMap) {
			if (i == "repeatSecret") {
				continue;
			}
			args[i] = valueMap[i];
		}
		return args;		
	},
	
	getRootServices: function (){
		var valueMap = this.getData();
		for (i in valueMap) {
			if (i == "rootServices") {
				return valueMap[i];
			}
		}
		return -1
	},
	
	getServerFromRootServices: function () {
		var rootservices = this.getRootServices();
		var server = rootservices.substring(0, rootservices.lastIndexOf('/'));
		return server;
	},
	
	validate: function() {
		var status = this.inherited("validate", arguments);
		if (!status.isValid) {
			status.message = this.messages.correctFields;
			this.setStatusCallback(status);
		} else {
			this.setStatusCallback(-1);
		}
		this.formButton.disabled = !status.overrideNext;
		
		return status;
	}
	
});


var WizardStep = com.ibm.team.repository.web.ui.wizard.WizardStep;
dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.EditFriendStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/EditFriend.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
		this.setupMessages = setupMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
	},	

	postCreate: function () {
		this.header =  new WizardStepHeader({stepTitle: this.stepTitle});
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		// Not using handleKeyUp with timeout because the validate is a simple check that there is content.
		// This way the user gets the Finish button much faster, but there are no errors to worry about.
		this.connect(this._editFriendTable,"onkeyup","validateChange");
		this.requiredInputs = [
			this._rootServicesInput, 
			this._titleInput
		]	
		var self = this;
		dojo.forEach(this.requiredInputs, function(elem) {
			if(elem) {
				self.connect(elem,"onchange","validateChange");
			}
		});

		if (this.friend.internal) {
			this._internalValue.innerHTML = this.messages.internalLabel;
		}
	}, 
		
	validate: function() {
		//Make sure none of the required input fields is blank
		dojo.forEach(this.requiredInputs, function(elem) {
			if(elem && elem.value ==  "") {
				return false;
			} 
		});
					
		return true;
	},
	
	validateChange: function () {
		this.triggerButtonStateUpdate();
	},
	
	saveUpdates: function () {
		var isValid = this.validate();
		if(!isValid) return;
		
		var args = {
			id: this.friend.id
		};
		
		if (this.friend.rootServicesURL != this._rootServicesInput.value) {
			args.rootServices = this._rootServicesInput.value;
		}
		if (this.friend.title != this._titleInput.value) {
			args.title = this._titleInput.value;
		}
		
		OAuthAdminClient.updateFriend(new SRH(this, "handleFriendUpdateSuccess", "displayServiceCallbackErrorMessage"), args);
	},
		
	handleFriendUpdateSuccess: function () {
		this.wizard.saveSuccess();		
	},
	
	displayServiceCallbackErrorMessage: function(errorObj) {
		this.header.displayServiceCallbackErrorMessage(errorObj);
	},
	
	canFinish: function(){
		var isValid = this.validate();
		return isValid && this.hasUpdates();
	}, 
	
	hasUpdates: function () {
		if (this.friend.rootServicesURL != this._rootServicesInput.value) {
			return true;
		}
		if (this.friend.title != this._titleInput.value) {
			return true;
		}
		return false;
	}
	

});


})();