/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURI");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");

dojo.require("com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigurePublicURIMessages");

(function() {

var WizardConfigurationStep = com.ibm.team.repository.web.ui.wizard.WizardConfigurationStep;

var REMOTE_SERVICE_NAME = "com.ibm.team.repository.service.internal.RepositoryRemoteService"
var messages;

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURI",  WizardConfigurationStep, {

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigurePublicURIMessages");
		this.implementationClassName = REMOTE_SERVICE_NAME;
		this.stepTitle = messages.wizardPageHeading;
		this.showTestButton = false;
		this.showRestoreButton = false;
		this.description = this.buildDescription();	
		this.STATUS.PENDING_CUSTOM = {test: false, revert: false, next: true, edit: true, code: "OK", message: 'goNext'};
		this.innerArgs = {headingLabel: messages.step1Label, heading: messages.step1Title};	
	},
	
	buildDescription: function() {
		var description = document.createElement("div");
		description.appendChild(this.buildDescriptionParagraph(messages.wizardPageDescription1));
		description.appendChild(this.buildDescriptionParagraph(messages.wizardPageDescription2));
		description.appendChild(this.buildDescriptionParagraph(messages.wizardPageDescription3));
		description.appendChild(this.buildDescriptionParagraph(messages.wizardPageDescription4));
		return description;
	},
	
	buildDescriptionParagraph: function(message) {
		var paragraph = document.createElement("p");
		paragraph.appendChild(document.createTextNode(message));
		return paragraph;
	},
	
	createInnerStepUI: function () {
		this.publicURLConfigUI = new ConfigurePublicURIInnerSteps();
		this.serviceContainer = this.publicURLConfigUI._configTable;
		this.configArea.appendChild(this.publicURLConfigUI.domNode);
	},
	
	//Custom override in order to remove a lot of the test connection wiring.
	//To include test connection code again, it's probably best to remove these 2 functions.
	_initializePropertiesAfterLoaded: function () {
		dojo.connect(this._config, "_markChanged", this, function() {
			this._setState(this.STATUS.PENDING_CUSTOM);
		});
		dojo.connect(this._config, "_checkUnchanged", this, function () {
				this._setState(this.STATUS.PENDING_CUSTOM);
			});
		dojo.connect(this._config, "_markInvalid", this, "_handleInvalidInput");
		this._setState(this.STATUS.PENDING_CUSTOM);
	},
	
	processNext: function() {
		this._initialLoadedState = this.STATUS.PENDING_CUSTOM;
		if (this.pending){
			var pendingProperties = this.pending.properties;
			for (var i = 0; i < pendingProperties.length; i++) {
				if (pendingProperties[i]._restartRequired) {
					serverRestart = true;
				}
				pendingProperties[i]._handleSuccessfulChange();
			}
		} 
		// TODO handle server restart message by passing in a "pending restart" state to setState
		this._setState(this.STATUS.PENDING_CUSTOM);
		this.pending = null;
		this.wizard.updateNavigationControls();
		this.wizard.processNext();
	}
	
	
	
});

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURIInnerSteps", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/ConfigurePublicURI.html"),

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ConfigurePublicURIMessages");
	}
	
});

var ConfigurePublicURIInnerSteps = com.ibm.team.repository.web.ui.internal.wizard.ConfigurePublicURIInnerSteps;


})();